"use strict";
let shouldInitializeExtension = true;

// 状態を表す定数
const TAB_STATE = {
  IDLE: 0,
  EXECUTE_ABILITY_SHORT_WAIT: 1,  // 1⇒2　回復後のアビ実行(短wait)
  EXECUTE_ABILITY_NORTMAL: 2,     // 2⇒2⇒リトライ終了時0 
  EXECUTE_JUDGE_RESULT: 3,        // 3⇒1　アビ実行結果が気絶の場合は、回復⇒アビ実行
  EXECUTE_RECOVERY_DP: 10,        // 10⇒0 任務回復薬を使用する
  EXECUTE_GET_ABILITY_FORM: 100,  // 編成アビ
  EXECUTE_GET_ABILITY_EQUIP: 101, // 所持アビ
  EXECUTE_GET_FORMATION: 110,     // 編成情報取得
  EXECUTE_GET_PROFILE: 200,       // ユーザ情報取得
};

console.debug("★content.js start ----------------");
checkState();
document.addEventListener("DOMContentLoaded", getBattleStatus);


// メインシナリオ
async function checkState() {
  const currentUrl = window.location.href;

  if (0) {
    // エラー画面の場合は、急がないので表示完了を待つ
    if (currentUrl.includes("battle_error.aspx")) {
      await new Promise(resolve => {
        document.addEventListener('DOMContentLoaded', resolve);
      });
      console.log("route1:", document.readyState);
    }
  }

  await checkOgiStatus("発:水鏡煌環", 5);

  console.log("route2:", document.readyState);
  const tabId = await getTabId();
  let tabState = await loadTabState(tabId);
  //console.log('tabState:', tabState);

  await checkDP(currentUrl, tabId);

  // タブ状態毎に処理を分割
  switch (tabState.status) {
    case TAB_STATE.EXECUTE_ABILITY_SHORT_WAIT:
      await excuteAbilityShortWait();
      break;
    case TAB_STATE.EXECUTE_ABILITY_NORTMAL:
      await excuteAbility(tabState.waitTime);
      break;
    case TAB_STATE.EXECUTE_JUDGE_RESULT:
      await judgeResult();
    case TAB_STATE.EXECUTE_RECOVERY_DP:
      await recoveryDP(currentUrl);
      break;
    default:
  }

  if (currentUrl.includes("battle_error.aspx")) {
    console.log("route3:", document.readyState);
    await handleBattleError(tabState);
  }

}


// 奥義状態判定
async function checkOgiStatus(ogiStatus, remain) {
  const battleState = await getBattleStatus();
  if (battleState && battleState.ogi1 && battleState.ogi1 == ogiStatus &&
    battleState.ogiRemain1 > 0 && battleState.ogiRemain1 <= remain) {
    console.log("リトライ終了:", battleState.ogi1);
    displayStatus(`水鏡残り${battleState.ogiRemain1}秒: リトライ終了`);
    clearTabState();
  }
}


// DP切れの場合は、任務回復薬使用画面に遷移する
async function checkDP(currentUrl, tabId) {
  if (currentUrl.includes("stamina_shortage.aspx")) {
    console.log("任務P切れ");
    const tabState = await loadTabState(tabId);
    tabState.status = TAB_STATE.EXECUTE_RECOVERY_DP; // 任務P回復実行
    tabState.retryCount = 1;
    tabState.url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fuser%2Fitem_detail.aspx%3Feid%3D100%26eic%3D1%26rnd%3D0&guid=ON";
    tabState.text = "任務P回復";
    await saveTabState(tabId, tabState);
    window.location.assign(tabState.url);
  }
}


// 任務回復薬使用
async function recoveryDP(currentUrl) {
  if (currentUrl.includes("item_detail.aspx")) {
    console.log("recoveryDP 回復薬ルート");
    // ボタンを取得
    var btnUse = document.getElementById('body_btnUse');
    console.log("任務P切れ(所持品画面):", btnUse);
    if (btnUse) {
      // ボタンをクリック
      await bsleep(500);
      btnUse.click();
    }
    await clearTabState();
  }
}


// 回復⇒攻撃
async function excuteAbilityShortWait() {
  console.log("excuteAbilityShortWait");
  const tabId = await getTabId();
  let tabState = await loadTabState(tabId);
  if (tabState.retryCount > 0) {
    const elapsedTime = await getElapsedTime(tabState);
    console.log("elapsedTime:", elapsedTime);
    await handleNormalExecution(tabState, await getWaitTime(settingId), elapsedTime);
  }
  else {
    // リトライ無し　⇒モバブラと同じ動作
    return;
  }
}


// 通常コマンド実行
async function excuteAbility(waitTime) {
  const tabId = await getTabId();
  let tabState = await loadTabState(tabId);
  console.log("excuteAbility rertyCount:", tabState.retryCount);

  if (tabState.retryCount <= 0) {
    // リトライ無し　⇒モバブラと同じ動作
    console.log("excuteAbility リトライ無し");
    return;
  }

  const elapsedTime = await getElapsedTime(tabState);
  console.log("elapsedTime:", elapsedTime);

  await handleNormalExecution(tabState, waitTime, elapsedTime);
  const tabStateToSave = {
    status: tabState.status,
    retryCount: tabState.retryCount,
    retryTotal: tabState.retryTotal,
    text: tabState.text,
    url: tabState.url,
    waitTime: tabState.waitTime
  };
  await saveTabState(tabId, tabStateToSave);

  if (tabState.retryCount > 0) {
    console.log("▶コマンド実行 retry:", tabState.retryCount);
    await window.location.assign(tabState.url);
  }
}


async function checkTabState() {
  const currentUrl = window.location.href;
  const tabId = await getTabId();
  let tabState = await loadTabState(tabId);
  const battleState = await getBattleStatus();

  console.log('Loaded tab state:', tabState);

  if (tabState.status && tabState.retryCount > 0) {
    const elapsedTime = await getElapsedTime(tabState);
    console.log("elapsedTime:", elapsedTime);

    if (battleState && battleState.ogi1 && battleState.ogi1 == "発:水鏡煌環" &&
      battleState.ogiRemain1 > 0 && battleState.ogiRemain1 <= 10) {
      console.log("リトライ終了:", battleState.ogi1);
      displayStatus(`水鏡残り${battleState.ogiRemain1}秒: リトライ終了`);
      clearTabState();
      return;
    }

    switch (tabState.status) {
      case TAB_STATE.EXECUTE_ABILITY_SHORT_WAIT:
        console.log("route1:", tabState.status);
        await handleNormalExecution(tabState, await getWaitTime(settingId), elapsedTime);
        break;

      case TAB_STATE.EXECUTE_ABILITY_NORTMAL:
        console.log("route2:", tabState.status);
        const aid = await extractAidString(tabState.url);
        if (aid == "3D50") {
          const tabStateToSave = {
            status: TAB_STATE.EXECUTE_JUDGE_RESULT, retryCount: tabState.retryCount,
            retryTotal: tabState.retryTotal, text: "通常弓", url: tabState.url, waitTime: tabState.waitTime
          };
          await saveTabState(tabId, tabStateToSave);
          judgeResult();
        } else {
          console.log("route2-2:", tabState.status);
          if (currentUrl.includes("battle_error.aspx")) {
            await handleBattleError(tabState);
            return;
          } else {
            await handleNormalExecution(tabState, tabState.waitTime, elapsedTime);
          }
        }
        break;

      default:
        console.log("route3:", tabState.status);
        judgeResult();
        break;
    }

    console.log("route4:", tabState.status);
    tabState = await loadTabState(tabId);

    if (tabState.retryCount <= 0) {
      tabState.status = TAB_STATE.IDLE;
    }

    const tabStateToSave = {
      status: tabState.status,
      retryCount: tabState.retryCount,
      retryTotal: tabState.retryTotal,
      text: tabState.text,
      url: tabState.url,
      waitTime: tabState.waitTime
    };
    await saveTabState(tabId, tabStateToSave);

    if (tabState.retryCount >= 0) {
      console.log("▶攻撃実行 retry:", tabState.retryCount);
      shouldInitializeExtension = false;
      await window.location.assign(tabState.url);
    }
  } else {
    console.log("route10 後衛リトライ無しorアビ以外:", tabState.status);
    if (currentUrl.includes("battle_error.aspx")) {
      await handleBattleError(tabState);
    }
  }
}


async function handleBattleError() {
  const message = extractNaviMessage();
  console.log("navi_message:", message);

  const tabId = await getTabId();
  const settingId = await getSettingId(tabId);
  let tabState = await loadTabState(tabId);

  if (message.includes("終了したよ")) {
    await clearTabState();
  } else if (message.includes("TPが足りない")) {
    displayMessage(`「自動任務」を実行します。`, true);
    await executeDuty(10, 360);
  } else if (message.includes("限られた回数") || message.includes("装備されていない") || message.includes("使用できないみたい")) {
    const aid = await extractAidString(tabState.url);
    console.log("aid:", aid);
    await clearStatus();
    await clearTabState();
    disableButton(aid);
  }

  // TP閾値チェック
  await checkTP();
}


// TPをチェックし閾値以下の場合は任務を実行する
async function checkTP() {
  const tpThreshold = await getTpThreshold();
  if (tpThreshold <= 0) {
    return;
  }

  let url;
  if (isCurrentTimeInBattle()) {
    // 決戦中
    url = "http://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fbattle%2Fbattle_top_sp.aspx%3Fvl%3D2%26rnd%3D926352&guid=ON";
  } else {
    //マイペ
    url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fmyp.aspx%3Frnd%3D750047&guid=ON";
  }

  const battleState = await getBattleStatus();
  if (battleState) {
    console.log("checkTP bs:", battleState);
    await bsleep(500);
    console.log("tpThreshold:", tpThreshold);
    if (battleState.tp <= tpThreshold) {
      await executeDuty(1, 360, url);
    }
  }
}


//　コンテンツ(アビ実行結果etc)を判定
async function judgeResult() {
  const tabId = await getTabId();
  await bsleep(50);
  const tabState = await loadTabState(tabId);
  console.log('stateCheckResult:', tabState);

  if (!tabState.status || tabState.retryCount < 0) {
    return;
  }
  const currentUrl = window.location.href;
  console.log("currentUrl:", currentUrl);

  // アビ実行結果取得
  if (currentUrl.includes("Ferror%2F400.aspx")) {
    // アク禁
    console.log("アク禁");
    await clearStatus();
    await clearTabState();
    bsleep(3000);
    return;
  }
  else if (currentUrl.includes("battle_error.aspx")) {
    await displayStatus(`${tabState.text} リトライ中 残り${tabState.retryCount}回`);

    var message = extractNaviMessage(); // エラーメッセージ取得
    if (0) { //debug
      const texts = ["攻撃を受けて気絶", "相手がみんな気絶",
        "限られた回数", "すでにこの決戦は終了したよ"];
      //const texts = ["相手がみんな気絶", "すでにこの決戦は終了したよ"];
      const randomNumber = Math.floor(Math.random() * texts.length);
      message = texts[randomNumber];
    }
    console.log("navi_message:", message);

    if (0) {
      // switch文に書き直し
      switch (true) {
        case /決戦は終了/.test(message):
          await clearTabState();
          await clearStatus();
          break;
        case /攻撃を受けて気絶/.test(message):
          console.log("攻撃失敗ルート: 回復実行");
          executeRecovery(tabState);
          return;
        case /TPが足りない/.test(message):
          await displayMessage(`「自動任務」を実行します。`, true);
          let url;
          // 使用例
          if (isCurrentTimeInBattle()) {
            // 決戦中
            url = "http://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fbattle%2Fbattle_top_sp.aspx%3Fvl%3D2%26rnd%3D926352&guid=ON";
          } else {
            //マイペ
            url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fmyp.aspx%3Frnd%3D750047&guid=ON";
          }
          await executeDuty(10, 360, url);
          break;
        default:
          console.log("switch default");
      }
    }

    if (message.includes("攻撃を受けて気絶")) {
      //回復処理　⇒アビ実行(state:1)
      await bsleep(800);
      console.log("攻撃失敗ルート: 回復実行");
      executeRecovery(tabState);
      return;
    }
    else if (message.includes("TPが足りない")) {
      await displayMessage("「自動任務」を実行します。", true);
      let url;
      // 使用例
      if (isCurrentTimeInBattle()) { // 決戦中
        url = "http://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fbattle%2Fbattle_top_sp.aspx%3Fvl%3D2%26rnd%3D926352&guid=ON";
      } else {
        url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fmyp.aspx%3Frnd%3D750047&guid=ON";
      }
      await executeDuty(10, 360, url);
      //await saveTabState(tabId, tabState);
    }
    else if (message.includes("相手がみんな気絶")) {
      await checkTP();
      console.log("敵全員気絶リトライroute:");
      //const battleState = await getBattleStatus();
      checkTP();
      await bsleep(500 + (1000 / tabState.retryCount));
      window.location.assign(tabState.url);
      return;
    }
    else if (message.includes("限られた回数") || message.includes("装備されていない")
      //message.includes("使用できないみたい")
    ) {
      // ボタン一時無効化
      const aid = await extractAidString(tabState.url);
      console.log("aid:", aid);
      if (aid != "3D10") {
        console.log("状態クリア");
        disableButton(aid);
      }
      await clearStatus();
      await clearTabState();
    }
    else if (message.includes("連続実行")) {
      bsleep(1000);
      window.location.assign(tabState.url);
      return;
    }
    else if (message.includes("終了したよ")) {
      if (0) {
        displayMessage(`「自動任務」を実行します。`, true);
        console.log("自動任務: 5回");
        await executeDuty(5, 350);
      }
      if (0) { //debug
        await bsleep(800);
        console.log("決戦時間外 回復debug");
        tabState.retryCount -= 1;
        await executeRecovery(tabState);
        //executeAbility(tabState);
        return;
      }
      if (0) {
        // ボタン一時無効化
        const aid = await extractAidString(tabState.url);
        console.log("aid:", aid);
        disableButton(aid);
      }
      if (0) {
        await bsleep(3000);
        const tabStateToSave = {
          status: tabState.status,
          retryCount: tabState.retryCount - 1,
          retryTotal: tabState.retryTotal,
          text: tabState.text,
          url: tabState.url,
          waitTime: tabState.waitTime
        };
        await saveTabState(tabId, tabStateToSave);
        location.reload();
      }
      if (0) {
        const aid = await extractAidString(tabState.url);
        disableButton(aid);
      }
      console.log("状態クリア");
      await clearTabState();
      await clearStatus();
      //return;
    }
    else {
      console.log("その他エラー", currentUrl);
      bsleep(1000);
      window.location.assign(tabState.url);
      return;
    }
  }
  else {
    console.log("その他URL:", currentUrl);
    tabState.retryCount -= 1;
    if (tabState.retryCount > 0) {
      displayStatus(`${tabState.text} リトライ中 残り${tabState.retryCount}回`);
      const tabStateToSave = {
        status: TAB_STATE.EXECUTE_JUDGE_RESULT,
        retryCount: tabState.retryCount,
        retryTotal: tabState.retryTotal,
        text: tabState.text,
        url: tabState.url,
        waitTime: tabState.waitTime
      };
      await saveTabState(tabId, tabStateToSave);
      console.log("▶攻撃実行 status 3⇒3");
      window.location.assign(tabState.url);
    }
    else {
      // status初期化
      await clearTabState();
    }
  }
}

if (shouldInitializeExtension) {
  //console.log('addEventListener: DOMContentLoaded');
  document.addEventListener("DOMContentLoaded", initializeButton);
  //initializeButton();
}


async function initializeButton() {
  console.log("ボタン表示処理開始");
  const tabId = await getTabId();
  const settingId = await getSettingId(tabId);
  //const buttonSize = await getButtonSize(settingId);

  if (settingId === TAB_STATE.EXECUTE_GET_ABILITY_FORM) {
    handleGetButtonData();
  } else if (settingId === TAB_STATE.EXECUTE_GET_ABILITY_EQUIP) {
    getAbilityFromEquip();
  } else if (settingId === TAB_STATE.EXECUTE_GET_FORMATION) {
    // カード編成
    getFormation();
  } else if (settingId === 200) {
    handlecheckLicense();
  } else {
    createStatusElement();
    createMenu(settingId);
    createStatusMessageElement();
    //console.log("★★initializeExtension end");
  }
}


// ボタン設定用編成情報取得
async function handleGetButtonData() {
  if (document.readyState === "complete" || document.readyState === "interactive") {
    getButtonsData();
  } else {
    document.addEventListener("DOMContentLoaded", getButtonsData);
  }
  await bsleep(500);
  chrome.runtime.sendMessage({ type: 'closeTab' });
}


// ユーザ情報取得
async function handlecheckLicense() {
  if (document.readyState === "complete" || document.readyState === "interactive") {
    getProfile();
  } else {
    document.addEventListener("DOMContentLoaded", getProfile)
  }
  await bsleep(200);
  chrome.runtime.sendMessage({ type: 'closeTab' });
}


// アビ実行
async function handleNormalExecution(tabState, waitTime, elapsedTime) {
  if (waitTime > elapsedTime) {
    const sleep = tabState.waitTime - elapsedTime;
    console.log("bsleep:", sleep);
    await bsleep(sleep);
  }
  tabState.retryCount -= 1;

  if (tabState.retryCount <= 0) {
    tabState.status = TAB_STATE.IDLE;
  }

}


// navi_message取得
function extractNaviMessage() {
  var messageElement = document.getElementById('navi_message');
  if (messageElement) {
    // 要素のテキストコンテンツを取得
    var message = messageElement.textContent.trim();
    return message;
  } else {
    return null;  // 要素が存在しない場合はnullを返すなど、適切な処理を行う
  }
}


// HTMLからユーザ情報を抽出する関数
async function getProfile() {
  var profile = [];

  // ユーザ名
  const element = document.querySelector('.bar-green');
  const uname = element.textContent.match(/(.+)の/)[1];
  // uid
  const uid = document.getElementById('body_divStatus').innerHTML.match(/u=(\d+)/)[1];
  console.log(uid);
  //gid
  const gid = document.getElementById('body_hlGuildName').href.match(/gid%(3D\d+)/)[1];
  console.log(gid);

  profile.push({ uname: uname, uid: uid, gid: gid });

  // ローカルストレージに保存
  await new Promise((resolve) => {
    chrome.storage.local.set({ 'Profile': profile }, function () {
      console.log('Data saved to local storage:', profile);
      resolve();
    });
  });
}


// リンクボタン処理
async function handleSelection(e, selection) {
  let n = selection || e.target;
  let label = n && n.getAttribute && n.getAttribute("data-action");

  // オプション設定取得
  const tabId = await getTabId();
  const settingId = await getSettingId(tabId);
  let waitTime = await getWaitTime(settingId);
  const recoveryOption = await getRecoveryOption(settingId);

  if (label) {
    // ボタンタイプを取得 (linkB1 -> B, linkL1 -> L, linkR1 -> R)
    const buttonType = label[4];
    const buttonSettings = await getButtonSettings(buttonType, settingId);

    if (label.includes("linkM")) {
      // メニューボタン処理
      processMenuButton(label);
    }
    else {
      // リンクボタン処理　labelに基づいて該当するボタンのURLを取得
      const buttonNumber = extractNumber(label);
      const jumpUrl = buttonNumber != null ? buttonSettings[buttonNumber - 1]?.url : null;
      let rUrl = await getRecoveryUrl(); // 回復用URL
      const buttonText = buttonNumber != null ? buttonSettings[buttonNumber - 1]?.text : null;
      let { repeatCount, interval } = await extractRepeatCountAndInterval(jumpUrl);

      console.log("label=", label, "repeatCount=", repeatCount, "interval=", interval);
      const shouldExecuteRecovery = recoveryOption === "on" && jumpUrl &&
        (
          jumpUrl.includes("3Faid%3D10") ||
          jumpUrl.includes("3Faid%3D11") ||
          jumpUrl.includes("3Faid%3D20%") ||
          (jumpUrl.includes("3Faid%3D10") && !/\d+$/.test(jumpUrl.slice(jumpUrl.indexOf("3Faid%3D10") + 10))) ||
          (jumpUrl.includes("3Faid%3D20") && !/\d+$/.test(jumpUrl.slice(jumpUrl.indexOf("3Faid%3D20") + 10))) ||
          (jumpUrl.includes("3Faid%3D40") && !/\d+$/.test(jumpUrl.slice(jumpUrl.indexOf("3Faid%3D40") + 10)))
          //(jumpUrl.match(/3Faid%3D(\d+)/) || [])[1] === "20" // 追加
        );

      const comboCheck = jumpUrl && /3Faid%3D2\d{3}/.test(jumpUrl);
      //console.log("comboCheck:", comboCheck);

      // ボタンのリンク有効時
      if (jumpUrl) {
        //console.log(`[${label}] jump start repeat=${repeatCount} interval=${interval}`);
        await clearTabState();
        // jumpUrlから#以降を削除
        let cleanJumpUrl = jumpUrl.split("#")[1] || jumpUrl;

        if (shouldExecuteRecovery) {
          const tabStateToSave = {
            status: TAB_STATE.EXECUTE_JUDGE_RESULT, // 回復前アビ実行
            retryCount: repeatCount,
            retryTotal: repeatCount,
            text: buttonText,
            url: cleanJumpUrl,
            waitTime: interval,
            waitTimeRecovery: waitTime
          };
          saveTabState(tabId, tabStateToSave);

          // 既に気絶画面の場合は、回復⇒攻撃実行
          // battleStatusで自身が気絶の場合も追加
          if (window.location.href.includes("battle_error.aspx") &&
            extractNaviMessage().includes("攻撃を受けて気絶")
          ) {
            console.log("▶前衛(気絶状態):回復⇒攻撃");
            executeRecovery(tabStateToSave);
            return;
          }
          else if (window.location.href.includes("battle_top_sp.aspx") &&
            getKzStatus()
          ) {
            // 決戦画面で、気絶中の場合
            console.log("▶前衛(気絶状態2):回復⇒攻撃");
            executeRecovery(tabStateToSave);
            return;
          }
          else {
            // アビを実行し、気絶などのエラー時はリトライする
            console.log("▶回復前アビ実行:", extractAidString(cleanJumpUrl));
            window.location.assign(cleanJumpUrl);
            return;
          }
        }
        else if (comboCheck) {
          // 後衛コンボチェックルート
          // 決戦画面の場合はコンボ状態チェック
        }

        if (repeatCount > 1) {
          if (jumpUrl.includes("duty_start.ashx")) {
            // 任務
            displayMessage(`「${buttonText}」を実行します。`, true);
            await executeDuty(repeatCount, interval);
          }
          else if (jumpUrl.includes("2Fgacha")) {
            //ガチャ
            displayMessage(`「${buttonText}」を実行します。`, true);
            console.log(`jumpUrl-repeat:`, cleanJumpUrl);
            for (let i = 1; i <= repeatCount; i++) {
              console.log(`jump i=${i}`);
              await fetch(cleanJumpUrl, { mode: 'no-cors', cache: 'no-cache' });
              await bsleep(interval);
            }
            displayMessage(`「${buttonText}」の実行が完了しました。`, true);
          }
          else {
            // 攻撃など
            displayMessage(`「${buttonText}」を実行します。`, true);
            console.log("▶アビ実行", cleanJumpUrl);
            // tab状態保存し、リロード
            const tabStateToSave = {
              status: TAB_STATE.EXECUTE_ABILITY_NORTMAL,
              retryCount: repeatCount - 1,
              retryTotal: repeatCount,
              text: buttonText,
              url: cleanJumpUrl,
              waitTime: interval
            };
            saveTabState(tabId, tabStateToSave);

            window.location.assign(cleanJumpUrl);
            //console.log("jumpUrl-attack repeat>1 end");
          }
        }
        else {
          const tabStateToSave = {
            status: TAB_STATE.EXECUTE_ABILITY_NORTMAL,
            //status: TAB_STATE.IDLE, ⇒相手全員気絶などでリトライできない
            retryCount: repeatCount - 1,
            retryTotal: repeatCount,
            text: buttonText,
            url: cleanJumpUrl,
            waitTime: interval
          };
          saveTabState(tabId, tabStateToSave);
          window.location.assign(cleanJumpUrl);
          console.log("▶アビ実行(no repeat):", tabStateToSave);
          return;
        }
      }
    }
  }
}


// メニュー
async function processMenuButton(label) {
  console.log("processMenuButton label:", label);
  await clearTabState();
  switch (label) {
    case "linkMPrevTab":
      chrome.runtime.sendMessage({ type: 'switchTab', direction: 'previous' });
      break;
    case "linkMNextTab":
      chrome.runtime.sendMessage({ type: 'switchTab', direction: 'next' });
      break;
    case "linkMOption":
      chrome.runtime.sendMessage({ type: 'openOptionsPage' });
      break;
    case "linkMDispButtons":
      console.log("表示ON/OFF");
      toggleHideLinkButtons();
      location.reload();
      break;
    case "linkMenu":
      createAdvancedMenu();
      break;
    case "linkMUrl":
      // クリップボードにテキストをコピー
      navigator.clipboard.writeText(window.location.href);
      break;
    case "linkMAbout":
      console.log("About");
      chrome.runtime.sendMessage({
        action: "openPopup"
      });
      break;
    default:
  }
}


// 任務実行
async function executeDuty(repeatCount, interval, nextUrl) {
  let url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fapi%2Fduty_start.ashx%3Fdi%3D811%26rnd%3D137405&guid=ON";
  let startTp = 0;
  let currentTp = 0;
  let currentDp = 0;

  const tabId = await getTabId();
  const tabState = await loadTabState(tabId);

  console.log("interval:", interval);

  for (let i = 1; i <= repeatCount; i++) {
    //console.log(`任務実行 i=${i}`);
    // 実行前時刻記録
    let tabState = {
      timestamp: new Date().getTime()
    }
    try {
      const response = await fetch(url, { cache: 'no-cache' });

      // レスポンスが成功かつHTMLであるか確認
      if (response.ok && response.headers.get('content-type').includes('text/html')) {
        const html = await response.text();
        const parser = new DOMParser();
        const doc = parser.parseFromString(html, 'text/html');
        //console.log(html);

        // 任務P
        const elementDp = doc.getElementById('currentDutyPNum');
        if (elementDp) {
          currentDp = elementDp.textContent.trim();
        } else {
          await bsleep(interval);
          continue;
        }

        //console.log("DP:", currentDp);
        if (currentDp < 10) {
          url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fuser%2Fitem_detail.aspx%3Feid%3D100%26eic%3D1%26rnd%3D0&guid=ON";
          const tabState = await loadTabState(tabId);
          tabState.status = 10; // 任務P回復実行
          tabState.retryCount = 1;
          tabState.url = url;
          tabState.text = "任務P回復";
          await saveTabState(tabId, tabState);
          console.log("▶任務P回復実行:");
          break;
        }

        // TP
        const elementTp = doc.getElementById('currentTpNum');
        currentTp = elementTp.textContent.trim();
        if (!startTp) {
          startTp = currentTp;
        }
        //console.log("TP:", currentTp);
        displayStatus(`任務実行中 ${i} TP=${currentTp}`);
        if (currentTp > startTp || currentTp > 90) {
          break;
        }
      } else {
        console.log('レスポンス異常。');
      }
    } catch (error) {
      console.log(`error route TP:${currentTp}, DP:${currentDp}`);

      //任務Pが足りない
      if (currentDp < 10 && currentDp > 0) {
        console.log("任務P回復ルート");
        url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fuser%2Fitem_detail.aspx%3Feid%3D100%26eic%3D1%26rnd%3D0&guid=ON";
        tabState.status = 10; // 任務P回復実行
        tabState.retryCount = 1;
        tabState.url = url;
        tabState.text = "任務P回復";
        await saveTabState(tabId, tabState);
        await window.location.assign(url);
        return;
      }
      else {
        console.log("ルート2");
        //break;
        //await window.location.assign(url);
      }
    }
    // 待機
    const currentTimestamp = new Date().getTime();
    const elapsedTime = currentTimestamp - tabState.timestamp;
    console.log("elapsedTime:", elapsedTime);
    if (elapsedTime < interval) {
      await bsleep(interval - elapsedTime);
    }
  }

  console.log(`TP:${currentTp}, DP:${currentDp}`);
  const currentUrl = window.location.href;

  //タイムスタンプ更新
  await bsleep(200);
  await saveTabState(tabId, tabState);
  console.log("currentUrl:", currentUrl);
  console.log("JumpUrl:", url);

  if (nextUrl) {
    window.location.assign(nextUrl);
  }
  else if (currentTp >= 90 && !currentUrl.includes("battle_error.aspx")) {
    await clearTabState();
    location.reload();
  }
  else {
    window.location.assign(url);
    await new Promise(resolve => {
      window.addEventListener('load', resolve, { once: true });
    });
  }
}


// aid取得関数(ログ出力用)
function extractAidString(url) {
  const match = url.match(/aid%3D(\d+)/);
  return match ? `3D${match[1]}` : null;
}


// 自動回復処理
async function executeRecovery(tabState) {
  const tabId = await getTabId();
  const rUrl = await getRecoveryUrl(); // 回復用URL
  const settingId = await getSettingId(tabId);
  const waitTimeRecovery = await getWaitTime(settingId);

  // 回復
  tabState.status = TAB_STATE.EXECUTE_JUDGE_RESULT;
  await saveTabState(tabId, tabState);
  console.log("▶自動回復実行:", waitTimeRecovery);
  window.location.assign(rUrl);
  await bsleep(waitTimeRecovery);

  // アビ実行
  console.log("▶アビ実行", extractAidString(tabState.url));
  tabState.retryCount -= 1;
  var tabStateToSave = {
    status: TAB_STATE.EXECUTE_JUDGE_RESULT,
    retryCount: tabState.retryCount,
    retryTotal: tabState.retryTotal,
    text: tabState.text,
    url: tabState.url,
    waitTime: tabState.interval
  };
  if (tabState.retryCount > 0) {
    tabStateToSave.status = tabState.status;
  }
  await saveTabState(tabId, tabStateToSave);
  await window.location.assign(tabState.url);

}
