document.addEventListener('DOMContentLoaded', function () {
    const buttonTextInputs = document.getElementById('buttonTextInputs');
    const recoveryOptionRadioGroup = document.getElementById('recoveryOptionRadioGroup');

    // 回復オプションのラジオボタンを生成
    const addRecoveryOptionRadio = (label, value, checked) => {
        const radio = document.createElement('input');
        radio.type = 'radio';
        radio.name = 'recoveryOption';
        radio.value = value;
        radio.checked = checked;

        const labelElement = document.createElement('label');
        labelElement.textContent = label;

        // ラジオボタンとラベルをテーブルに追加
        const row = document.createElement('tr');
        const radioCell = document.createElement('td');
        const labelCell = document.createElement('td');
        radioCell.appendChild(radio);
        labelCell.appendChild(labelElement);
        row.appendChild(radioCell);
        row.appendChild(labelCell);
        recoveryOptionRadioGroup.appendChild(row);
    };

    // 回復ONと回復OFFのラジオボタンを生成
    chrome.storage.sync.get({ recoveryOption: 'on' }, obj => {
        addRecoveryOptionRadio('回復ON', 'on', obj.recoveryOption === 'on');
        addRecoveryOptionRadio('回復OFF', 'off', obj.recoveryOption === 'off');
    });


    // 保存ボタンのクリックイベント
    document.getElementById('saveButton').addEventListener('click', function () {
        // 回復オプション
        const recoveryOption = document.querySelector('input[name="recoveryOption"]:checked').value;

        // 回復後待機時間
        const waitTime = document.getElementById('waitTime').value;

        // ボタンテキストとURLを取得
        const buttonSettings = [];

        // ボタン数取得の共通処理
        const getButtonCount = (prefix) => {
            const buttonCountElement = document.getElementById(`buttonCount${prefix}`);
            return buttonCountElement ? parseInt(buttonCountElement.value, 10) : 0;
        };

        const processButtons = (prefix) => {
            const buttonCount = getButtonCount(prefix);
            for (let i = 1; i <= buttonCount; i++) {
                const buttonTextElement = document.getElementById(`button${prefix}${i}Text`);
                const buttonText = (buttonTextElement && buttonTextElement.value) || `リンク${prefix.charAt(prefix.length - 1)}${i}`;
                const buttonUrlElement = document.getElementById(`button${prefix}${i}Url`);
                const buttonUrl = (buttonUrlElement && buttonUrlElement.value) || '';
                buttonSettings.push({ text: buttonText, url: buttonUrl });
            }
        };

        processButtons('B');
        processButtons('L');
        processButtons('R');

        const buttonCountB = getButtonCount("B");
        const buttonCountL = getButtonCount("L");
        const buttonCountR = getButtonCount("R");

        // 待機時間、ボタン設定、回復オプションを保存
        chrome.storage.sync.set({
            buttonCountB: buttonCountB,
            buttonCountL: buttonCountL,
            buttonCountR: buttonCountR,
            waitTime: waitTime,
            buttonSettings: buttonSettings,
            recoveryOption: recoveryOption
        }, function () {
            console.log('Settings saved:', { buttonCountB, buttonCountL, buttonCountR, waitTime, buttonSettings, recoveryOption });
            displayStatusMessage('保存しました', true);
        });
    });


    // 保存時のステータスメッセージ表示
    function displayStatusMessage(message, success) {
        const statusMessageElement = document.getElementById('statusMessage');
        statusMessageElement.textContent = message;
        statusMessageElement.style.color = success ? 'green' : 'red';
        setTimeout(() => {
            statusMessageElement.textContent = '';
        }, 3000);
    }


    // デフォルトのボタン設定を生成
    function getDefaultButtonSettings(prefix) {
        const buttonSettings = [];
        for (let i = 1; i <= 10; i++) {
            buttonSettings.push({ text: `リンク${prefix}${i}`, url: '' });
        }
        return buttonSettings;
    }

    // ボタン数を元に動的にボタン入力項目を生成する関数
    async function generateButtonInputs(prefix, buttonCount) {
        // 現在の入力項目をクリア
        const buttonTextInputs = document.getElementById(`buttonTextInputs${prefix}`);
        buttonTextInputs.innerHTML = '';
        //console.log(buttonTextInputs); // デバッグ用

        // 新しいボタンテキストとURLの入力フィールドを動的に生成
        for (let i = 1; i <= buttonCount; i++) {
            const buttonLabel = document.createElement('label');
            buttonLabel.textContent = `ボタン${i}`;
            buttonLabel.classList.add('option-button-label');

            const labelText = document.createElement('input');
            labelText.type = 'text';
            labelText.id = `button${prefix}${i}Text`;
            labelText.placeholder = `${buttonLabel.textContent}のテキスト`;

            const urlInput = document.createElement('textarea');
            urlInput.id = `button${prefix}${i}Url`;
            urlInput.placeholder = `${buttonLabel.textContent}のURL`;

            const row = document.createElement('tr');
            const labelAndTextCell = document.createElement('td');
            const urlCell = document.createElement('td');

            labelAndTextCell.appendChild(buttonLabel);
            labelAndTextCell.appendChild(labelText);
            row.appendChild(labelAndTextCell);
            buttonTextInputs.appendChild(row);

            const rowUrl = document.createElement('tr');
            const urlCellInner = document.createElement('td');
            urlCellInner.appendChild(urlInput);
            rowUrl.appendChild(urlCellInner);
            buttonTextInputs.appendChild(rowUrl);

        }

        // 保存されているデータがあれば設定
        const data = await chrome.storage.sync.get({ [`button${prefix}Settings`]: getDefaultButtonSettings(prefix) });
        if (data[`button${prefix}Settings`]) {
            for (let i = 1; i <= buttonCount; i++) {
                const buttonTextElement = document.getElementById(`button${prefix}${i}Text`);
                const buttonUrlElement = document.getElementById(`button${prefix}${i}Url`);

                if (buttonTextElement && buttonUrlElement) {
                    buttonTextElement.value = data[`button${prefix}Settings`][i - 1].text;
                    buttonUrlElement.value = data[`button${prefix}Settings`][i - 1].url;
                } else {
                    console.error(`Button text or URL element not found for button ${i}.`);
                }
            }
        }
        // 追加: 非同期処理が完了したことをログで出力
        console.log(`After generateButtonInputs: buttonTextInputs${prefix}:`, document.getElementById(`buttonTextInputs${prefix}`));
    }


    // ボタン数を元に動的にボタン入力項目を生成する関数
    function generateButtonInputsOld(prefix, buttonCount) {
        // 現在の入力項目をクリア
        const buttonTextInputs = document.getElementById(`buttonTextInputs${prefix}`);
        buttonTextInputs.innerHTML = '';
        //console.log(buttonTextInputs); // デバッグ用

        // 新しいボタンテキストとURLの入力フィールドを動的に生成
        for (let i = 1; i <= buttonCount; i++) {

            const buttonLabel = document.createElement('label');
            buttonLabel.textContent = `ボタン${i}`;
            buttonLabel.classList.add('option-button-label');

            const labelText = document.createElement('input');
            labelText.type = 'text';
            labelText.id = `button${prefix}${i}Text`;
            labelText.placeholder = `${buttonLabel.textContent}のテキスト`;

            const urlInput = document.createElement('textarea');
            urlInput.id = `button${prefix}${i}Url`;
            urlInput.placeholder = `${buttonLabel.textContent}のURL`;

            const row = document.createElement('tr');
            const labelAndTextCell = document.createElement('td');
            const urlCell = document.createElement('td');

            labelAndTextCell.appendChild(buttonLabel);
            labelAndTextCell.appendChild(labelText);
            row.appendChild(labelAndTextCell);
            buttonTextInputs.appendChild(row); /* ★ */

            const rowUrl = document.createElement('tr');
            const urlCellInner = document.createElement('td');
            urlCellInner.appendChild(urlInput);
            rowUrl.appendChild(urlCellInner);
            buttonTextInputs.appendChild(rowUrl);

            // 保存されているデータがあれば設定
            chrome.storage.sync.get({ [`button${prefix}Settings`]: getDefaultButtonSettings(prefix) }, function (data) {
                if (data[`button${prefix}Settings`][i - 1]) {
                    labelText.value = data[`button${prefix}Settings`][i - 1].text;
                    urlInput.value = data[`button${prefix}Settings`][i - 1].url;
                }
            });
        }
    }

    // 保存されている待機時間とボタンのテキストを取得して表示
    chrome.storage.sync.get({
        buttonCountB: 5,
        buttonCountL: 10,
        buttonCountR: 10,
        waitTime: 300,
        buttonSettings: getDefaultButtonSettings(),
        recoveryOption: 'on'
    }, function (data) {
        console.log('Retrieved data from storage:', data);

        document.getElementById('waitTime').value = data.waitTime;

        document.getElementById('buttonCountB').value = data.buttonCountB || 5;
        document.getElementById('buttonCountL').value = data.buttonCountL || 10;
        document.getElementById('buttonCountR').value = data.buttonCountR || 10;

        // 回復オプションの値を設定
        const recoveryOptionRadio = document.querySelector(`input[name="recoveryOption"][value="${data.recoveryOption}"]`);
        if (recoveryOptionRadio) {
            recoveryOptionRadio.checked = true;
        } else {
            console.error('Recovery option radio button not found.');
        }
        //console.log(`buttonSettings:`, data.buttonSettings);

        // 以下の部分での呼び出し
        ['B', 'L', 'R'].forEach(async prefix => {
            // DEBUG: `buttonTextInputs` のログを追加
            const buttonTextInputs = document.getElementById(`buttonTextInputs${prefix}`);
            console.log(`Before getElementById: buttonTextInputs${prefix}:`, buttonTextInputs);

            await setButtonSettings(prefix, data);

            // 追加: setButtonSettings の後でログを出力
            console.log(`After setButtonSettings: buttonTextInputs${prefix}:`, buttonTextInputs);

            await generateButtonInputs(prefix, data[`buttonCount${prefix}`]);
        });
    });


    // ボタンテキストとURLを設定
    async function setButtonSettings(prefix, data) {
        const buttonCount = data[`buttonCount${prefix}`];
        const buttonSettings = data.buttonSettings || getDefaultButtonSettings(prefix);
        console.log(`buttonSettings:`, buttonSettings);
        //const el = document.getElementById("buttonB1Text");
        //console.log(`debug buttonB1Text:`, el); /* ★ */
  
        for (let i = 1; i <= buttonCount; i++) {
            const buttonTextElement = document.getElementById(`button${prefix}${i}Text`);
            const buttonUrlElement = document.getElementById(`button${prefix}${i}Url`);
            //console.log(`document:`, document);
            //console.log(`buttonTextElement id=button${prefix}${i}Text:`, buttonTextElement);
            //console.log(`buttonUrlElement:`, buttonUrlElement);

            if (buttonTextElement && buttonUrlElement) {
                buttonTextElement.value = buttonSettings[i - 1].text;
                buttonUrlElement.value = buttonSettings[i - 1].url;
            } else {
                //console.error(`Button text or URL element not found for button: button${prefix}${i}Text`);
            }
        }
    }

});
